#!/usr/bin/env node
var program = require('commander');
var iothub = require('azure-iothub');
var package = require('../package.json');
var Preferences = require('preferences');
var prefs = new Preferences('iothub');

function printSingleDevice(deviceInfo) {
    console.log();
    console.log('Device Id: ' + deviceInfo.deviceId);
    console.log('Status: ' + deviceInfo.status);
    console.log('Primary Key: ' + deviceInfo.authentication.SymmetricKey.primaryKey);
    console.log('Secondary Key: ' + deviceInfo.authentication.SymmetricKey.secondaryKey);    
}

function printDevices(data, json) {
    if (json) {
        // Output as JSON format    
        console.log(JSON.stringify(data, null, 2));
    } else {
        // Check if data is a single device or an array
        if (Object.prototype.toString.call(data) === '[object Array]' ) {
            for (var i = 0; i < data.length; i++) {
                printSingleDevice(data[i]);
            }                
        } else {
            printSingleDevice(data);
        }
    }
}

// Connect command
program
    .command('connect <connectionstring>')        
    .action(function(connectionstring) {        
        try {
            registry = iothub.Registry.fromConnectionString(connectionstring);
            prefs.connectionstring = connectionstring;
            console.log('Connected to ' + registry._transport._config.hubName + '.');
        } catch (err) {
            console.log('Could not connect to an Azure IoT Hub with the provided conenction string.');
        }
    });

// Disconnect command
program
    .command('disconnect')        
    .action(function() {        
        prefs.connectionstring = null;
        console.log('Disconnect.');        
    });

// List devices command
program
    .command('list')
    .option('-j, --json', 'Sets JSON as output format')    
    .action(function(options) {
        if (!prefs.connectionstring) {
            console.log('No connection found. Please connect to an Azure IoT Hub first using "iothub connect <connectionstring>".');
            return;
        }

        registry = iothub.Registry.fromConnectionString(prefs.connectionstring);
        registry.list(function(err, devices, res) {
            console.log(devices.length + ' devices found.');
            printDevices(devices, options.json);
        });
    });

// Add device command
program
    .command('add <deviceId>')
    .option('-s, --status <status>', 'Initial status of the device when created')
    .option('-r, --statusReason <statusReason>', 'Reason, why the device has been enabled or disabled')
    .option('-j, --json', 'Sets JSON as output format')
    .action(function(deviceId, options) {
        if (!prefs.connectionstring) {
            console.log('No connection found. Please connect to an Azure IoT Hub first using "iothub connect <connectionstring>".');
            return;
        }

        registry = iothub.Registry.fromConnectionString(prefs.connectionstring);
        var device = new iothub.Device(null);
        device.deviceId = deviceId;   

        if (options.status)       
            device.status = options.status;        
        if (options.statusReason)
            device.statusReason = options.statusReason;

        registry.create(device, function(err, deviceInfo, res) {
            if (deviceInfo) {
                // New device created
                console.log('New device "' + device.deviceId + '" has been created successfully.');
                printDevices(deviceInfo, options.json);
            }  

            if (err) {
                // Check if device has already been existant
                registry.get(device.deviceId, function (err, deviceInfo, res) {
                    if (err) {
                        // Could not find existant device
                        console.log('ERROR: ' + err);
                    } else {
                        console.log('Device "' + device.deviceId + '" has already been existant.');
                        printDevices(deviceInfo, options.json);
                    }
                }); 
            }
        })
    });

// Delete device command
program
    .command('delete <deviceId>')
    .action(function(deviceId) {
        if (!prefs.connectionstring) {
            console.log('No connection found. Please connect to an Azure IoT Hub first using "iothub connect <connectionstring>".');
            return;
        }

        registry = iothub.Registry.fromConnectionString(prefs.connectionstring);
        registry.delete(deviceId, function(err, deviceInfo, res) {
            if (err) {
                // Check if device has already been existant
                registry.get(deviceId, function (err, deviceInfo, res) {
                    if (err) {
                        // Could not find existant device
                        console.log('ERROR: Device "' + deviceId + '" could not be found.');
                    } else {
                        console.log('ERROR: Device "' + deviceId + '" could not be deleted.');
                    }
                }); 
            } else {
                // Device deleted
                console.log('Device "' + deviceId + '" has been deleted successfully.');
            }
        })
    });

// Version
program.version(package.version);

// Parse inputs
program.parse(process.argv);